@@ -16,7 +16,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-__version__ = "3.1.0"
+__version__ = "3.2.0"
 
 from typing import TYPE_CHECKING
 
@@ -82,9 +82,11 @@ _import_structure = {
     "model_mixin": [
         "EmbeddingAdaptersMixin",
         "InvertibleAdaptersMixin",
+        "InvertibleAdaptersWrapperMixin",
         "ModelAdaptersMixin",
         "ModelWithHeadsAdaptersMixin",
     ],
+    "models.albert": ["AlbertAdapterModel"],
     "models.auto": [
         "ADAPTER_MODEL_MAPPING",
         "MODEL_WITH_HEADS_MAPPING",
@@ -95,10 +97,12 @@ _import_structure = {
         "BartAdapterModel",
         "BartModelWithHeads",
     ],
+    "models.beit": ["BeitAdapterModel"],
     "models.bert": [
         "BertAdapterModel",
         "BertModelWithHeads",
     ],
+    "models.bert_generation": ["BertGenerationAdapterModel"],
     "models.deberta": ["DebertaAdapterModel"],
     "models.debertaV2": ["DebertaV2AdapterModel"],
     "models.distilbert": [
@@ -109,6 +113,7 @@ _import_structure = {
         "GPT2AdapterModel",
         "GPT2ModelWithHeads",
     ],
+    "models.gptj": ["GPTJAdapterModel"],
     "models.mbart": [
         "MBartAdapterModel",
         "MBartModelWithHeads",
@@ -129,7 +134,7 @@ _import_structure = {
     "trainer": ["AdapterTrainer", "Seq2SeqAdapterTrainer"],
     "training": [
         "AdapterArguments",
-        "MultiLingAdapterArguments",
+        "setup_adapter_training",
     ],
     "utils": [
         "ADAPTER_CACHE",
@@ -198,23 +203,28 @@ if TYPE_CHECKING:
     from .model_mixin import (
         EmbeddingAdaptersMixin,
         InvertibleAdaptersMixin,
+        InvertibleAdaptersWrapperMixin,
         ModelAdaptersMixin,
         ModelWithHeadsAdaptersMixin,
     )
+    from .models.albert import AlbertAdapterModel
     from .models.auto import ADAPTER_MODEL_MAPPING, MODEL_WITH_HEADS_MAPPING, AutoAdapterModel, AutoModelWithHeads
     from .models.bart import BartAdapterModel, BartModelWithHeads
+    from .models.beit import BeitAdapterModel
     from .models.bert import BertAdapterModel, BertModelWithHeads
+    from .models.bert_generation import BertGenerationAdapterModel
     from .models.deberta import DebertaAdapterModel
     from .models.debertaV2 import DebertaV2AdapterModel
     from .models.distilbert import DistilBertAdapterModel, DistilBertModelWithHeads
     from .models.gpt2 import GPT2AdapterModel, GPT2ModelWithHeads
+    from .models.gptj import GPTJAdapterModel
     from .models.mbart import MBartAdapterModel, MBartModelWithHeads
     from .models.roberta import RobertaAdapterModel, RobertaModelWithHeads
     from .models.t5 import T5AdapterModel, T5ModelWithHeads
     from .models.vit import ViTAdapterModel
     from .models.xlm_roberta import XLMRobertaAdapterModel, XLMRobertaModelWithHeads
     from .trainer import AdapterTrainer, Seq2SeqAdapterTrainer
-    from .training import AdapterArguments, MultiLingAdapterArguments
+    from .training import AdapterArguments, setup_adapter_training
     from .utils import (
         ADAPTER_CACHE,
         AdapterInfo,
