@@ -10,11 +10,18 @@ import torch
 from torch import nn
 
 from .composition import AdapterCompositionBlock, Fuse, Stack, parse_composition
-from .configuration import AdapterConfig, AdapterConfigBase, AdapterFusionConfig, get_adapter_config_hash
+from .configuration import (
+    ADAPTER_CONFIG_MAP,
+    AdapterConfig,
+    AdapterConfigBase,
+    AdapterFusionConfig,
+    get_adapter_config_hash,
+)
 from .context import AdapterSetup, ForwardContext
 from .hub_mixin import PushAdapterToHubMixin
 from .layer import AdapterLayer, AdapterLayerBase
 from .loading import AdapterFusionLoader, AdapterLoader, PredictionHeadLoader, WeightsLoader
+from .lora import LoRALayer
 from .modeling import Adapter, GLOWCouplingBlock, NICECouplingBlock
 from .prefix_tuning import PrefixTuningPool, PrefixTuningShim
 from .utils import EMBEDDING_FILE, TOKENIZER_PATH, inherit_doc
@@ -32,7 +39,8 @@ class InvertibleAdaptersMixin:
         self.invertible_adapters = nn.ModuleDict(dict())
 
         # Make sure config is wrapped
-        self.config = wrap_config(self.config)
+        if hasattr(self, "config"):
+            self.config = wrap_config(self.config)
 
     def add_invertible_adapter(self, adapter_name: str):
         """
@@ -44,6 +52,7 @@ class InvertibleAdaptersMixin:
         """
         if adapter_name in self.invertible_adapters:
             raise ValueError(f"Model already contains an adapter module for '{adapter_name}'.")
+        embedding_size = getattr(self.config, "embedding_size", self.config.hidden_size)
         adapter_config = self.config.adapters.match(
             adapter_name,
             config_type=AdapterConfig,
@@ -52,13 +61,13 @@ class InvertibleAdaptersMixin:
         if adapter_config and adapter_config["inv_adapter"]:
             if adapter_config["inv_adapter"] == "nice":
                 inv_adap = NICECouplingBlock(
-                    [[self.config.hidden_size]],
+                    [[embedding_size]],
                     non_linearity=adapter_config["non_linearity"],
                     reduction_factor=adapter_config["inv_adapter_reduction_factor"],
                 )
             elif adapter_config["inv_adapter"] == "glow":
                 inv_adap = GLOWCouplingBlock(
-                    [[self.config.hidden_size]],
+                    [[embedding_size]],
                     non_linearity=adapter_config["non_linearity"],
                     reduction_factor=adapter_config["inv_adapter_reduction_factor"],
                 )
@@ -95,6 +104,222 @@ class InvertibleAdaptersMixin:
         return hidden_states
 
 
+class InvertibleAdaptersWrapperMixin:
+    """
+    Mixin for Transformer models supporting invertible adapters in a child module. When applying this mixin, set
+    `invertible_adapters_base_name` to the name of the child module that includes `InvertibleAdaptersMixin`.
+    """
+
+    invertible_adapters_base_name = ""
+
+    @property
+    def invertible_adapters_base(self):
+        return getattr(self, self.invertible_adapters_base_name, None)
+
+    @property
+    def invertible_adapters(self):
+        if self.invertible_adapters_base is not None:
+            return self.invertible_adapters_base.invertible_adapters
+        return None
+
+    def add_invertible_adapter(self, adapter_name: str):
+        """
+        Adds an invertible adapter module for the adapter with the given name. If the given adapter does not specify an
+        invertible adapter config, this method does nothing.
+
+        Args:
+            adapter_name (str): The name of the adapter for which to add an invertible adapter module.
+        """
+        if self.invertible_adapters_base is not None:
+            self.invertible_adapters_base.add_invertible_adapter(adapter_name)
+
+    def delete_invertible_adapter(self, adapter_name: str):
+        if self.invertible_adapters_base is not None:
+            self.invertible_adapters_base.delete_invertible_adapter(adapter_name)
+
+    def get_invertible_adapter(self):
+        if self.invertible_adapters_base is not None:
+            return self.invertible_adapters_base.get_invertible_adapter()
+        return None
+
+    def enable_invertible_adapters(self, adapter_names):
+        if self.invertible_adapters_base is not None:
+            self.invertible_adapters_base.enable_invertible_adapters(adapter_names)
+
+    def invertible_adapters_forward(self, hidden_states, rev=False):
+        if self.invertible_adapters_base is not None:
+            return self.invertible_adapters_base.invertible_adapters_forward(hidden_states, rev=rev)
+        return hidden_states
+
+
+class EmbeddingAdaptersMixin:
+    """Mixin for Transformer models adding support for dynamically switching embeddings."""
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.loaded_embeddings = {}
+        self._active_embedding = "default"
+
+        # Make sure config is wrapped
+        if hasattr(self, "config"):
+            self.config = wrap_config(self.config)
+
+    def load_embeddings(self, path: str, name: str):
+        """
+        Load a saved embedding from the given path. If the embedding was saved with a tokenizer it is returned
+
+        Args:
+            path: the path to the saved embedding
+            name: the name the embedding should be loaded as
+
+        Returns: a tokenizer if it ws saved with the embedding otherwise None
+
+        """
+        from ..models.auto.tokenization_auto import AutoTokenizer
+
+        if name in self.loaded_embeddings:
+            raise ValueError("An embedding with the name {} already exists".format(name))
+        tokenizer = None
+        tokenizer_path = os.path.join(path, TOKENIZER_PATH)
+        if os.path.isdir(tokenizer_path):
+            tokenizer = AutoTokenizer.from_pretrained(tokenizer_path)
+
+        embedding_path = os.path.join(path, EMBEDDING_FILE)
+        if not os.path.isfile(embedding_path):
+            raise FileNotFoundError("No embeddings found at {}".format(embedding_path))
+        weights = torch.load(embedding_path)
+
+        self.loaded_embeddings[name] = nn.Embedding.from_pretrained(weights)
+        self.set_active_embeddings(name)
+        return tokenizer
+
+    def add_embeddings(self, name, tokenizer, reference_embedding=None, reference_tokenizer=None, embedding_dim=None):
+        """
+        Add a new embedding to the model. If a reference embedding and reference tokenizer are provided tokens in the
+        present in both tokenizers are initialized to the embedding in the reference_embedding.
+
+        Args:
+            name: the name of the embedding
+            tokenizer: the tokenizer determining the vocab of the embedding
+            reference_embedding:
+                the reference embedding to use for initializing the embeddings of tokens present in the newly created
+                embedding
+            reference_tokenizer: the tokenizer providing the vocab for the reference embedding
+            embedding_dim:
+                the dimension of the embeddings (if None the embedding_size, or if this doesn't exist the hidden_size,
+                from the config is used)
+        """
+        if name in self.loaded_embeddings:
+            raise ValueError("An embedding with the name {} already exists".format(name))
+        if embedding_dim is not None:
+            embedding_size = embedding_dim
+        else:
+            embedding_size = getattr(self.config, "embedding_size", self.config.hidden_size)
+        embedding = nn.Embedding(len(tokenizer), embedding_size)
+        # Use same initialization as base Transformer model
+        embedding.weight.data.normal_(mean=0.0, std=0.02)
+        if embedding.padding_idx is not None:
+            embedding.weight.data[embedding.padding_idx].zero_()
+        embedding.requires_grad_(False)
+        if (reference_embedding is not None and reference_tokenizer is None) or (
+            reference_tokenizer is not None and reference_embedding is None
+        ):
+            raise KeyError(
+                "Reference embedding and reference tokenizer are required to use initialize embeddings from reference"
+                " embedding"
+            )
+        if reference_embedding is not None and reference_tokenizer is not None:
+            tokens = set(tokenizer.get_vocab().keys()) & set(reference_tokenizer.get_vocab().keys())
+            reference_vocab = reference_tokenizer.get_vocab()
+            vocab = tokenizer.get_vocab()
+            for t in tokens:
+                idx_reference = reference_vocab[t]
+                idx = vocab[t]
+                embedding.weight[idx] = (
+                    self.loaded_embeddings[reference_embedding].weight[idx_reference].detach().clone()
+                )
+        embedding.train(False)
+        self.loaded_embeddings[name] = embedding
+        self.set_active_embeddings(name)
+
+    def delete_embeddings(self, name):
+        """
+        Deletes the embedding with the given name
+
+        Args:
+            name: The name of the embedding that should be deleted
+
+        """
+        if name not in self.loaded_embeddings:
+            raise ValueError("No embedding with name {}".format(name))
+        if self.active_embeddings == name:
+            logger.warning("The active embedding is deleted. Setting the default embedding as active.")
+            self.set_active_embeddings("default")
+        del self.loaded_embeddings[name]
+
+    def save_embeddings(self, path, name, tokenizer=None):
+        """
+        Saves the embedding with the given name. If a tokenizer is passed as well the tokenizer is saved together with
+        the embedding.
+
+        Args:
+            path: The path where the embedding should be saved
+            name: The name of the embedding that should be saved
+            tokenizer: optionally a tokenizer to save with the embedding (default is None)
+
+        """
+        if self.active_embeddings == name:
+            self.loaded_embeddings[name] = self.get_input_embeddings()
+        os.makedirs(path, exist_ok=True)
+        embedding_path = os.path.join(path, EMBEDDING_FILE)
+        torch.save(self.loaded_embeddings[name].weight, embedding_path)
+        if tokenizer:
+            tokenizer_path = os.path.join(path, TOKENIZER_PATH)
+            tokenizer.save_pretrained(tokenizer_path)
+
+    def set_active_embeddings(self, name):
+        """
+        Sets the active embedding for the forward pass of the model
+
+        Args:
+            name: The name of the embedding that should be used
+
+        """
+        self.loaded_embeddings[self.active_embeddings] = self.get_input_embeddings()
+        self.set_input_embeddings(self.loaded_embeddings[name])
+        self.config.vocab_size = self.loaded_embeddings[name].num_embeddings
+        self._active_embedding = name
+
+    @property
+    def active_embeddings(self):
+        return self._active_embedding
+
+
+class EmbeddingAdaptersWrapperMixin:
+    def load_embeddings(self, path: str, name: str):
+        return self.base_model.load_embeddings(path, name)
+
+    def add_embeddings(self, name, tokenizer, reference_embedding=None, reference_tokenizer=None):
+        return self.base_model.add_embeddings(name, tokenizer, reference_embedding, reference_tokenizer)
+
+    def delete_embeddings(self, name):
+        return self.base_model.delete_embeddings(name)
+
+    def save_embeddings(self, path, name, tokenizer=None):
+        return self.base_model.save_embeddings(path, name, tokenizer)
+
+    def set_active_embeddings(self, name):
+        return self.base_model.set_active_embeddings(name)
+
+    @property
+    def active_embeddings(self):
+        return self.base_model.active_embeddings
+
+    @property
+    def loaded_embeddings(self):
+        return self.base_model.loaded_embeddings
+
+
 class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
     """Mixin for transformer models adding support for loading/ saving adapters."""
 
@@ -104,9 +329,7 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
             self.model_name = config.name_or_path
         else:
             self.model_name = None
-        self.loaded_embeddings = {}
-        self.shared_parameters = nn.ModuleDict()
-        self._active_embedding = "default"
+        self.base_model.shared_parameters = nn.ModuleDict()
 
         # Make sure config is wrapped
         self.config = wrap_config(self.config)
@@ -126,12 +349,13 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
 
         # Initialize adapters from config
         for adapter_name in self.config.adapters:
-            self.apply_to_adapter_layers(lambda i, layer: layer.add_adapter(adapter_name, i))
+            self._add_adapter_weights(adapter_name)
         # Initialize fusion from config
         for fusion_name in self.config.adapters.fusions:
             self.apply_to_adapter_layers(lambda i, layer: layer.add_fusion_layer(fusion_name))
 
-        self.loaded_embeddings["default"] = self.get_input_embeddings()
+        if isinstance(self, EmbeddingAdaptersMixin):
+            self.loaded_embeddings["default"] = self.get_input_embeddings()
 
     # These methods have to be implemented by every deriving class:
 
@@ -160,11 +384,11 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
         adapter_setup = parse_composition(adapter_setup)
         self.apply_to_adapter_layers(lambda i, layer: layer.enable_adapters(adapter_setup, True, False))
         for adapter_name in adapter_setup:
-            if adapter_name in self.shared_parameters:
-                for param in self.shared_parameters[adapter_name].values():
+            if adapter_name in self.base_model.shared_parameters:
+                for param in self.base_model.shared_parameters[adapter_name].values():
                     param.requires_grad = True
 
-        if isinstance(self, InvertibleAdaptersMixin):
+        if isinstance(self, InvertibleAdaptersMixin) or isinstance(self, InvertibleAdaptersWrapperMixin):
             self.enable_invertible_adapters(adapter_setup.flatten())
         # use the adapters to be trained by default in every forward pass
         self.set_active_adapters(adapter_setup)
@@ -210,7 +434,7 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
         self.set_active_adapters(adapter_setup)
 
     def set_shared_parameters(self, param):
-        self.shared_parameters = param
+        self.base_model.shared_parameters = param
 
     def set_active_adapters(
         self, adapter_setup: Union[list, AdapterCompositionBlock], skip_layers: Optional[List[int]] = None
@@ -228,9 +452,12 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
             for adapter_name in adapter_setup.flatten():
                 if adapter_name not in self.config.adapters.adapters:
                     raise ValueError(
-                        f"No adapter with name '{adapter_name}' found. Please make sure that all specified adapters are correctly loaded."
+                        f"No adapter with name '{adapter_name}' found. Please make sure that all specified adapters"
+                        " are correctly loaded."
                     )
 
+        # Make sure LoRA is reset
+        self.reset_adapter()
         self.config.adapters.active_setup = adapter_setup
         self.config.adapters.skip_layers = skip_layers
 
@@ -239,44 +466,47 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
         Adds a new adapter module of the specified type to the model.
 
         Args:
-
-            adapter_name (str): The name of the adapter module to be added. config (str or dict or AdapterConfigBase,
-            optional): The adapter configuration, can be either:
+            adapter_name (str): The name of the adapter module to be added.
+            config (str or dict or AdapterConfigBase, optional): The adapter configuration, can be either:
 
                 - the string identifier of a pre-defined configuration dictionary
                 - a configuration dictionary specifying the full config
                 - if not given, the default configuration for this adapter type will be used
-            overwrite_ok (bool, optional): Overwrite an adapter with the same name if it exists. By default (False), an
-            exception is thrown. set_active (bool, optional): Set the adapter to be the active one. By default (False),
+            overwrite_ok (bool, optional):
+                Overwrite an adapter with the same name if it exists. By default (False), an
+            exception is thrown. set_active (bool, optional):
+                Set the adapter to be the active one. By default (False),
             the adapter is added but not activated.
         """
-        if isinstance(config, dict):
-            config = AdapterConfigBase.load(config)  # ensure config is ok and up-to-date
+        config = AdapterConfigBase.load(config)  # ensure config is ok and up-to-date
         # In case adapter already exists and we allow overwriting, explicitly delete the existing one first
         if overwrite_ok and adapter_name in self.config.adapters:
             self.delete_adapter(adapter_name)
         self.config.adapters.add(adapter_name, config=config)
         try:
-            self.apply_to_adapter_layers(lambda i, layer: layer.add_adapter(adapter_name, i))
-            # PHM Layer
-            if self.config.adapters.match(adapter_name, AdapterConfig, location_key="phm_layer"):
-                self._add_shared_parameters(adapter_name, config)
-            # Prefix Tuning
-            for module in self.modules():
-                if isinstance(module, PrefixTuningPool):
-                    module.confirm_prefix(adapter_name)
-            if isinstance(self, InvertibleAdaptersMixin):
-                self.add_invertible_adapter(adapter_name)
+            self._add_adapter_weights(adapter_name)
         except ValueError as ex:
             self.delete_adapter(adapter_name)
             raise ex
         if set_active:
             self.set_active_adapters(adapter_name)
 
-    def _add_shared_parameters(self, adapter_name, adapter_config: AdapterConfig):
-        self.shared_parameters[adapter_name] = (
-            list(self.get_adapter(adapter_name)[0].values())[0].adapter_down[0].init_shared_parameters()
-        )
+    def _add_adapter_weights(self, adapter_name: str):
+        """Helper method that performs the actual parameter additions when adding a new adapter."""
+        self.apply_to_adapter_layers(lambda i, layer: layer.add_adapter(adapter_name, i))
+        # PHM Layer
+        if self.config.adapters.match(adapter_name, AdapterConfig, location_key="phm_layer"):
+            adapter_module = list(self.get_adapter(adapter_name)[0].values())[0]
+            # if multiple adapters with same location key exist they are returned as a modulelist
+            if isinstance(adapter_module, nn.ModuleList):
+                adapter_module = adapter_module[0]
+            self.base_model.shared_parameters[adapter_name] = adapter_module.adapter_down[0].init_shared_parameters()
+        # Prefix Tuning
+        for module in self.modules():
+            if isinstance(module, PrefixTuningPool):
+                module.confirm_prefix(adapter_name)
+        if isinstance(self, InvertibleAdaptersMixin) or isinstance(self, InvertibleAdaptersWrapperMixin):
+            self.add_invertible_adapter(adapter_name)
 
     def add_fusion(self, adapter_names: Union[Fuse, list], adapter_fusion_config=None, override_kwargs=None):
         warnings.warn(
@@ -342,7 +572,10 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
             return
         del self.config.adapters.adapters[adapter_name]
         self.apply_to_adapter_layers(lambda i, layer: layer.delete_adapter(adapter_name))
-        if isinstance(self, InvertibleAdaptersMixin):
+        # PHM Layer
+        if adapter_name in self.base_model.shared_parameters:
+            del self.base_model.shared_parameters[adapter_name]
+        if isinstance(self, InvertibleAdaptersMixin) or isinstance(self, InvertibleAdaptersWrapperMixin):
             self.delete_invertible_adapter(adapter_name)
         # Reset active adapters if this was the only active adapter
         if self.active_adapters == Stack(adapter_name):
@@ -549,6 +782,7 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
         Args:
             save_directory (str): Path to a directory where the adapters should be saved.
         """
+        os.makedirs(save_directory, exist_ok=True)
         for name in self.config.adapters:
             adapter_config = self.config.adapters.get(name)
             h = get_adapter_config_hash(adapter_config)
@@ -572,6 +806,7 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
         Args:
             save_directory (str): Path to a directory where the AdapterFusion layers should be saved.
         """
+        os.makedirs(save_directory, exist_ok=True)
         for name in self.config.adapters.fusions:
             adapter_fusion_config = self.config.adapters.get_fusion(name)
             h = get_adapter_config_hash(adapter_fusion_config)
@@ -596,157 +831,46 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
         This method is called by the ``ForwardContext`` at the beginning of the forward pass.
         """
         # some warnings if we don't use available adapters
-        active_adapters = getattr(self, "active_adapters", None) or AdapterSetup.get_context()
+        active_adapters = getattr(self, "active_adapters", None) or AdapterSetup.get_context_adapter_setup()
         if not active_adapters:
             if self.has_adapters():
                 logger.warning("There are adapters available but none are activated for the forward pass.")
             return
 
         context.adapters_parallelized = False
+        # Check if already parallelized in encoder
+        adapter_input_parallelized = kwargs.pop("adapter_input_parallelized", None)
+        if adapter_input_parallelized:
+            if active_adapters.parallel_channels > 1:
+                context.adapters_parallelized = True
         # Add the shared parameters for the active adapters to the context
         context.shared_parameters = {
-            name: param for name, param in self.shared_parameters.items() if name in active_adapters.flatten()
+            name: param
+            for name, param in self.base_model.shared_parameters.items()
+            if name in active_adapters.flatten()
         }
 
-        # Prefix tuning
-        input_tensor = kwargs.get("input_ids", None)
-        if input_tensor is None:
-            input_tensor = kwargs.get("decoder_input_ids", None)
-        if input_tensor is None:
-            input_tensor = kwargs.get("attention_mask", None)
-        if input_tensor is None:
-            input_tensor = args[0]
-        context.prefix_states = self.base_model.prefix_tuning(input_tensor.shape[0])
-
-    def load_embeddings(self, path: str, name: str):
-        """
-        Load a saved embedding from the given path. If the embedding was saved with a tokenizer it is returned
-
-        Args:
-            path: the path to the saved embedding
-            name: the name the embedding should be loaded as
-
-        Returns: a tokenizer if it ws saved with the embedding otherwise None
-
-        """
-        from ..models.auto.tokenization_auto import AutoTokenizer
-
-        if name in self.loaded_embeddings:
-            raise ValueError("An embedding with the name {} already exists".format(name))
-        tokenizer = None
-        tokenizer_path = os.path.join(path, TOKENIZER_PATH)
-        if os.path.isdir(tokenizer_path):
-            tokenizer = AutoTokenizer.from_pretrained(tokenizer_path)
-
-        embedding_path = os.path.join(path, EMBEDDING_FILE)
-        if not os.path.isfile(embedding_path):
-            raise FileNotFoundError("No embeddings found at {}".format(embedding_path))
-        weights = torch.load(embedding_path)
-
-        self.loaded_embeddings[name] = nn.Embedding.from_pretrained(weights)
-        self.set_active_embeddings(name)
-        return tokenizer
-
-    def add_embeddings(self, name, tokenizer, reference_embedding=None, reference_tokenizer=None, embedding_dim=None):
-        """
-        Add a new embedding to the model. If a reference embedding and reference tokenizer are provided tokens in the
-        present in both tokenizers are initialized to the embedding in the reference_embedding.
-
-        Args:
-            name: the name of the embedding
-            tokenizer: the tokenizer determining the vocab of the embedding
-            reference_embedding:
-                the reference embedding to use for initializing the embeddings of tokens present in the newly created
-                embedding
-            reference_tokenizer: the tokenizer providing the vocab for the reference embedding
-            embedding_dim: the dimension of the embeddings (if None the hidden_size from the config is used)
-
-        """
-        if name in self.loaded_embeddings:
-            raise ValueError("An embedding with the name {} already exists".format(name))
-        if embedding_dim is None:
-            embedding_dim = self.config.hidden_size
-        embedding = nn.Embedding(tokenizer.vocab_size, embedding_dim)
-        embedding.requires_grad_(False)
-        if (reference_embedding is not None and reference_tokenizer is None) or (
-            reference_tokenizer is not None and reference_embedding is None
-        ):
-            raise KeyError(
-                "Reference embedding and reference tokenizer are required to use initialize embeddings from reference embedding"
-            )
-        if reference_embedding is not None and reference_tokenizer is not None:
-            tokens = set(tokenizer.get_vocab().keys()) & set(reference_tokenizer.get_vocab().keys())
-            reference_vocab = reference_tokenizer.get_vocab()
-            vocab = tokenizer.get_vocab()
-            for t in tokens:
-                idx_reference = reference_vocab[t]
-                idx = vocab[t]
-                embedding.weight[idx] = self.loaded_embeddings[reference_embedding].weight[idx_reference].clone()
-        embedding.train(False)
-        self.loaded_embeddings[name] = embedding
-        self.set_active_embeddings(name)
-
-    def delete_embeddings(self, name):
-        """
-        Deletes the embedding with the given name
-
-        Args:
-            name: The name of the embedding that should be deleted
-
-        """
-        if name not in self.loaded_embeddings:
-            raise ValueError("No embedding with name {}".format(name))
-        if self.active_embeddings == name:
-            logger.warning("The active embedding is deleted. Setting the default embedding as active.")
-            self.set_active_embeddings("default")
-        del self.loaded_embeddings[name]
-
-    def save_embeddings(self, path, name, tokenizer=None):
-        """
-        Saves the embedding with the given name. If a tokenizer is passed as well the tokenizer is saved together with
-        the embedding.
-
-        Args:
-            path: The path where the embedding should be saved
-            name: The name of the embedding that should be saved
-            tokenizer: optionally a tokenizer to save with the embedding (default is None)
-
-        """
-        if self.active_embeddings == name:
-            self.loaded_embeddings[name] = self.get_input_embeddings()
-        os.makedirs(path, exist_ok=True)
-        embedding_path = os.path.join(path, EMBEDDING_FILE)
-        torch.save(self.loaded_embeddings[name].weight, embedding_path)
-        if tokenizer:
-            tokenizer_path = os.path.join(path, TOKENIZER_PATH)
-            tokenizer.save_pretrained(tokenizer_path)
-
-    def set_active_embeddings(self, name):
-        """
-        Sets the active embedding for the forward pass of the model
-
-        Args:
-            name: The name of the embedding that should be used
-
-        """
-        self.loaded_embeddings[self.active_embeddings] = self.get_input_embeddings()
-        self.set_input_embeddings(self.loaded_embeddings[name])
-        self._active_embedding = name
-
-    @property
-    def active_embeddings(self):
-        return self._active_embedding
+        context.prefix_states = self.base_model.prefix_tuning(*args, **kwargs)
+        # Adapter gating and attention outputs
+        context.output_adapter_gating_scores = kwargs.get("output_adapter_gating_scores", False)
+        context.output_adapter_fusion_attentions = kwargs.get("output_adapter_fusion_attentions", False)
+        context.adapter_gating_scores = defaultdict(dict)
+        context.adapter_fusion_attentions = defaultdict(dict)
 
     def get_fusion_regularization_loss(self):
-        reg_loss = 0.0
+        reg_loss = None
 
         target = torch.zeros((self.config.hidden_size, self.config.hidden_size)).fill_diagonal_(1.0).to(self.device)
         for i, layer in self.iter_layers():
             for module in layer.modules():
                 if isinstance(module, AdapterLayer):
                     for _, layer_fusion in module.adapter_fusion_layer.items():
-                        if hasattr(layer_fusion, "value"):
-                            reg_loss += 0.01 * (target - layer_fusion.value.weight).pow(2).sum()
+                        if hasattr(layer_fusion, "value") and layer_fusion.value.weight.requires_grad:
+                            layer_reg_loss = 0.01 * (target - layer_fusion.value.weight).pow(2).sum()
+                            if reg_loss is None:
+                                reg_loss = layer_reg_loss
+                            else:
+                                reg_loss += layer_reg_loss
 
         return reg_loss
 
@@ -759,20 +883,105 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
 
         Returns:
             dict: A nested dictionary containing the weights of the adapter. The dictionary is structured as follow:
-            {<layer id>: {<module location>: <nn.Module>}}.
+            {<layer id>: {<module location>: <nn.Module>}}. <layer id> = -1 indicates global/ shared weights.
         """
         destination = defaultdict(dict)
 
+        # global weights are saved at index -1
+        if name in self.base_model.shared_parameters:
+            destination[-1]["shared"] = self.base_model.shared_parameters[name]
+        if (
+            isinstance(self, InvertibleAdaptersMixin) or isinstance(self, InvertibleAdaptersWrapperMixin)
+        ) and name in self.invertible_adapters:
+            destination[-1]["invertible"] = self.invertible_adapters[name]
+
         # use a custom index to ensure numbering is from 0 to N layers
         for i, (_, layer) in enumerate(self.iter_layers()):
             for module in layer.modules():
                 if isinstance(module, AdapterLayerBase):
                     adapter_module = module.get_adapter(name)
                     if adapter_module is not None:
-                        destination[i][module.location_key] = adapter_module
+                        # location_key might already be added before -> concat to ModuleList
+                        if module.location_key in destination[i]:
+                            old_module = destination[i][module.location_key]
+                            if isinstance(old_module, nn.ModuleList):
+                                old_module.append(adapter_module)
+                            else:
+                                destination[i][module.location_key] = nn.ModuleList([old_module, adapter_module])
+                        else:
+                            destination[i][module.location_key] = adapter_module
 
         return dict(destination)
 
+    def adapter_summary(self, as_dict=False) -> Union[str, dict]:
+        """
+        Returns a string summary of all adapters currently added to the model. Each entry in the summary table has the
+        following attributes:
+
+            - name: the name of the adapter
+            - architecture: the architectural base of the adapter
+            - #param: the number of parameters of the adapter
+            - %param: the number of parameters of the adapter relative to the full model
+            - active: whether the adapter is active
+            - train: whether the adapter weights are enabled for training
+        """
+        # table header
+        header = ["name", "architecture", "#param", "%param", "active", "train"]
+        # rows containing adapter info
+        rows = []
+        # fill in data for adapters
+        for name, config_name in self.config.adapters.adapters.items():
+            if config_name in self.config.adapters.config_map:
+                config = self.config.adapters.config_map.get(config_name, None)
+            else:
+                config = ADAPTER_CONFIG_MAP.get(config_name, None)
+            if isinstance(config, str):
+                config = ADAPTER_CONFIG_MAP[config]
+            row = {"name": name, "architecture": config.get("architecture", None) or "bottleneck"}
+            weights = self.get_adapter(name)
+            row["active"] = self.active_adapters is not None and name in self.active_adapters.flatten()
+            # count parameters
+            no_params = 0
+            train = True
+            for _, module_dict in weights.items():
+                for _, module in module_dict.items():
+                    no_params += sum(p.numel() for p in module.parameters())
+                    train &= all(p.requires_grad for p in module.parameters())
+            row["#param"] = no_params
+            row["train"] = train
+            rows.append(row)
+        # count no. of parameters in base network
+        model_no_params = sum(p.numel() for p in self.base_model.parameters())
+        model_no_params -= sum([r["#param"] for r in rows])
+        # add %param info
+        for row in rows:
+            row["%param"] = row["#param"] / model_no_params * 100
+        # add full model info
+        rows.append(
+            {
+                "name": "Full model",
+                "#param": model_no_params,
+                "%param": 100.0,
+                "train": not getattr(self.base_model, "model_frozen", False),
+            }
+        )
+
+        if as_dict:
+            return rows
+        else:
+            # print
+            total_length = 80
+            header_format = "{:<25}{:<15}{:>12}{:>12}{:>8}{:>8}"
+            row_format = "{:<25}{:<15}{:>12,}{:>12.3f}{:>8}{:>8}"
+            s = ["=" * total_length]
+            s.append(header_format.format(*map(lambda x: x.title(), header)))
+            s.append("-" * total_length)
+            for row in rows:
+                s.append(row_format.format(*[row.get(h, "") for h in header]))
+            s.insert(len(s) - 1, "-" * total_length)
+            s.append("=" * total_length)
+            return "\n".join(s)
+
     def eject_prefix_tuning(self, name: str):
         """
         Converts the prefix tuning with the given name from the reparameterized form into the flat form.
@@ -785,6 +994,26 @@ class ModelAdaptersMixin(PushAdapterToHubMixin, ABC):
                 if name in module.prefix_tunings:
                     module.prefix_tunings[name].eject()
 
+    def merge_adapter(self, name: str):
+        """
+        Merges the weights of the given LoRA module with the Transformer weights as described in the paper.
+
+        Args:
+            name (str): LoRA module to merge.
+        """
+        for module in self.modules():
+            if isinstance(module, LoRALayer):
+                if name in module.loras:
+                    module.merge_adapter(name)
+
+    def reset_adapter(self):
+        """
+        Resets weights of a LoRA module merged using `model.merge_adapter(name)`.
+        """
+        for module in self.modules():
+            if isinstance(module, LoRALayer):
+                module.reset_adapter()
+
 
 @inherit_doc
 class ModelWithHeadsAdaptersMixin(ModelAdaptersMixin):
@@ -796,11 +1025,6 @@ class ModelWithHeadsAdaptersMixin(ModelAdaptersMixin):
         super().__init__(config, *args, **kwargs)
         self._convert_to_flex_head = False
 
-    def set_shared_parameters(self, param):
-        self.shared_parameters = param
-        if self.base_model is not self:
-            self.base_model.shared_parameters = self.shared_parameters
-
     def iter_layers(self) -> Iterable[Tuple[int, nn.Module]]:
         """
         Iterates over all layers of the model.
@@ -931,15 +1155,22 @@ class ModelWithHeadsAdaptersMixin(ModelAdaptersMixin):
         meta_dict: dict = None,
         custom_weights_loaders: Optional[List[WeightsLoader]] = None,
     ):
-        if with_head:
-            if custom_weights_loaders is None:
-                custom_weights_loaders = []
-            custom_weights_loaders.append(PredictionHeadLoader(self, error_on_missing=False))
-        super().save_all_adapters(
-            save_directory,
-            meta_dict=meta_dict,
-            custom_weights_loaders=custom_weights_loaders,
-        )
+        os.makedirs(save_directory, exist_ok=True)
+        for name in self.config.adapters:
+            adapter_config = self.config.adapters.get(name)
+            h = get_adapter_config_hash(adapter_config)
+            save_path = join(save_directory, name)
+            if meta_dict:
+                meta_dict.update({"config_id": h})
+            else:
+                meta_dict = {"config_id": h}
+            self.save_adapter(
+                save_path,
+                name,
+                meta_dict=meta_dict,
+                with_head=with_head,
+                custom_weights_loaders=custom_weights_loaders,
+            )
 
     def save_adapter_fusion(
         self,
@@ -996,6 +1227,7 @@ class ModelWithHeadsAdaptersMixin(ModelAdaptersMixin):
         super().load_adapter_fusion(adapter_fusion_name_or_path, load_as, custom_weights_loaders, set_active)
 
     def save_all_heads(self, save_directory):
+        os.makedirs(save_directory, exist_ok=True)
         for head_name in self.heads:
             save_path = join(save_directory, head_name)
             self.save_head(save_path, head_name)
@@ -1015,35 +1247,3 @@ class ModelWithHeadsAdaptersMixin(ModelAdaptersMixin):
             return super().get_adapter(name)
         else:
             return self.base_model.get_adapter(name)
-
-    def load_embeddings(self, path: str, name: str):
-        if self.base_model is self:
-            return super().load_embeddings(path, name)
-        else:
-            return self.base_model.load_embeddings(path, name)
-
-    def save_embeddings(self, path, name, tokenizer=None):
-        if self.base_model is self:
-            return super().save_embeddings(path, name, tokenizer)
-        else:
-            return self.base_model.save_embeddings(path, name, tokenizer)
-
-    def add_embeddings(self, name, tokenizer, reference_embedding=None, reference_tokenizer=None, embedding_dim=None):
-        if self.base_model is None:
-            return super().add_embeddings(name, tokenizer, reference_embedding, reference_tokenizer, embedding_dim)
-        else:
-            return self.base_model.add_embeddings(
-                name, tokenizer, reference_embedding, reference_tokenizer, embedding_dim
-            )
-
-    def set_active_embeddings(self, name):
-        if self.base_model is None:
-            return super().set_active_embeddings(name)
-        else:
-            return self.base_model.set_active_embeddings(name)
-
-    def delete_embeddings(self, name):
-        if self.base_model is None:
-            return super().delete_embeddings(name)
-        else:
-            return self.base_model.delete_embeddings(name)
